/*******************************************************************************

  MEDIDOR DE BATIDAS CARDIACAS

  C/ PIC 12f675 e LCD 16X2 (usando LPLEX a 3 fios)


  Autor: Cludio Lrios

  Data:  08/10/2017

  Objetivo: Medir o nmero de batidas cardiacas por minuto

  Uso didtico

  Este arquivo  parte integrante do blog PicSource (Lrios.Tecnologia.WS)

*******************************************************************************/
#include <12F675.h>
#device adc=10  //10 bits (no usado)
#use delay(clock=4000000)//frequencia 4 mhz osc.interno
#fuses NOWDT,HS,/*INTRC_IO,*/ NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT//configurao
#rom 0x2100 = {"MONITOR CARDIAC0 BLOG  PICSOURCE AGUARDE...       N. BATIDAS:     "}

//bytes
#byte  trisio = 0x85
#byte  gpio   = 0x05
#byte  tmr0   = 0x01

//pinos do lcd (metodo Lariosplex 3 FIOS)
#bit DB76      = 0x5.1  //pino 6 db7=direto db6=resistor 10k +cap 10nF ao gnd
#bit DB54      = 0x5.0  //pino 7 db5=direto db4=resistor 10k +cap 10nF ao gnd
#bit E_RS      = 0x5.2  //pino 5
#bit in_sensor = 0x05.3  //pino 4 sw

#bit wpu0      = 0x95.0
#bit wpu1      = 0x95.1
#bit gppu      = 0x81.7
#bit t0if      = 0x0b.2
#bit t0ie      = 0x0b.5
#bit gie       = 0x0b.7

//defines para LPLEX
#define vcarga 500 // por tentativa e erro (entre 200 a 500)
#define vdescarga 500//idem
#define v_delay_clk 2  // de 2 a 50 conforme LCD

//variveis globais
unsigned int1 RS,flag1;// flags
unsigned int8 a,res;
unsigned int16 timer;

/*******************************************************************************
                          SUBROTINAS
*******************************************************************************/
void descarregar(){ DB76=0;DB54=0;E_RS=0; delay_us(vdescarga);}
void t_carga(){delay_us(vcarga);}
void clk(){ E_RS=1; delay_cycles(v_delay_clk); E_RS=0;}//gera clock par Lcd

/*******************************************************************************
                          SUBROTINAS de LCD
*******************************************************************************/


/*******************************************************************************
                         ENVIA NIBBLE
*******************************************************************************/
void env_nible(unsigned int8 nib){
    descarregar();//descarrega capacitores
    nib&=0x0f;//limita
    if(nib&0x01)DB54=1;//testa bit 0
    if(nib&0x04)DB76=1;//testa bit 2
    if(RS)E_RS=1;else E_RS=0;//comando ou dado
    t_carga();
    if(nib&0x02)DB54=1; else DB54=0;//testa bit 1
    if(nib&0x08)DB76=1; else DB76=0;//testa bit 3
    clk();//clock
}

/*******************************************************************************
                          ENVIA DADOS PARA O LCD
*******************************************************************************/
void env_lcd_d(unsigned int8 dado){
 RS=1;//indica que enviar 'dados'
 env_nible(dado>>4);//high nibble primeiro
 env_nible(dado);
 delay_ms(4);
}

/*******************************************************************************
                         ENVIA COMANDOS PARA LCD
*******************************************************************************/
void env_lcd_c(unsigned int8 comando){
 RS=0;//indica que enviar 'comandos'
 env_nible(comando>>4);//high nibble primeiro
 env_nible(comando);
 delay_ms(4);
}

/*******************************************************************************
                         INICIALIZAO DO LCD
*******************************************************************************/
void Lcd_Init(){
  unsigned int8 a;
   descarregar();//descarrega capacitores
   RS=0; //enviar comando
   delay_ms(40);//delay inicial
   for(a=0;a<3;a++) { env_nible(3);delay_ms(5);}//envia 3x 0x03
   env_nible(2);//envia 0x02
   delay_ms(5);//delay 5 ms
   env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
   env_lcd_c(0x06);// no shift
   env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
  // env_lcd_c(0x0f);//Display ON, Cursor OFF, Blink On
   env_lcd_c(0x01);//limpa tela
}

/*******************************************************************************
                         DELAY
*******************************************************************************/
void delay_200ms(){ delay_ms(200);}
void delay_dbc(){delay_ms(20);}
void delay_1seg(){
   for(a=0;a<5;a++){//mantem a tela apagada por um segundo
    delay_200ms();
   }
}

/*******************************************************************************
                           ROTINA MAIN
*******************************************************************************/
void main() {
   setup_adc_ports(NO_ANALOGS);//desliga adc
   setup_comparator(NC_NC_NC_NC);//desliga comparadores analgicos
   setup_timer_0 (RTCC_DIV_1|RTCC_INTERNAL);
   trisio=0b111000;//ajusta entradas e sadas
   gpio=0;//zera todas sadas do PIC
   descarregar();//descarga dos capacitores do Lariosplex
   Lcd_Init();//inicializa Lcd
   for(a=0;a<16;a++)env_lcd_d(read_eeprom(a));//nome do aparelho na eeprom
   env_lcd_c(0xc0);//posiciona na linha 2
   for(a=0;a<16;a++)env_lcd_d(read_eeprom(a+17));//envia p/ lcd 'blog picsource'
   delay_1seg();//mostra mensagem no lcd por um segundo
   delay_1seg();//mostra mensagem no lcd por mais um segundo
   gie=0;//chave geral de interrupes

/*******************************************************************************
                           ROTINA DE REPETIO
*******************************************************************************/
   for(;;){

   #define VALOR_15SEG  60000  //valor para 15 segundos

     env_lcd_c(1); //limpa tela
     for(a=0;a<16;a++)env_lcd_d(read_eeprom(a+32));//envia para lcd 'AGUARDE...'
     set_timer0(0);//reseta timer 0
     t0if=0;//reseta flag de over flow do timer 0
     res=0;//limpa resultado
     timer=VALOR_15SEG;//valor para obter uma leitura de pulsos por 15 segundos
     flag1=0;//resta flag

/*******************************************************************************
           CONTAGEM DOS PULSOS DO SENSOR
*******************************************************************************/
     do{
        set_timer0(23);//carga do timer para 250 useg
        t0if=0;//reseta flag de over flow do timer 0
        if((!in_sensor) && (!flag1)) {flag1=1; res++;}//testa/incrementa resultado
        if(in_sensor)flag1=0;//testa/reseta flag
        while(!t0if);//espera completar 250 us
        timer--;//decrementa contador auxiliar de tempo
        if(!timer)break;//se atingiu tempo de 15 segundos sai do loop 'do/while'
     } while(1);
     
/*******************************************************************************
          APRESENTA RESULTADO NO LCD
*******************************************************************************/
     env_lcd_c(0x80);//comeo da linha 1 do lcd
     for(a=0;a<16;a++)env_lcd_d(read_eeprom(a+50));//envia para lcd 'N.BATIDAS:'
     res=res*4;//multiplica resultado por 4 
     env_lcd_c(0x8C);//posiciona local para o resultado 
     if(res>99){// se maior que 99 usar casa das centenas
     env_lcd_d(res/100 |0x30);//envia centenas
     }
     res=res%100;//coloca resto da diviso
     env_lcd_d(res/10| 0x30);//envia dezenas
     res=res%10;//coloca resto da diviso
     env_lcd_d(res|0x30);//envia unidades
     delay_ms(5000);//tempo entre leituras de 5 segundos
    }//for(;;)
}//main()

/*******************************************************************************
                           FIM DO PROGRAMA
*******************************************************************************/

